/**************************************************************************/
/*! 
    @file     NFCReader.cpp
    @author   jose maria sabater
 
    This example is for read the FreestyleLibre tag
*/
/**************************************************************************/

/* la lectura  del tag de Abbott la haremos utilizando el 
CR95HF incluido en el BM019 de Solutions Cubed */

#include "Arduino.h"
#include "NFCReader.h"


NFCReader::NFCReader(int SSPinNFC, int IRQPinNFC)
{
	SSPin = SSPinNFC;
	IRQPin= IRQPinNFC;
	_NFCReady=0;
}

void NFCReader::NFCsetup(){
	
	pinMode(IRQPin, OUTPUT);
    digitalWrite(IRQPin, HIGH); // Wake up pulse
    pinMode(SSPin, OUTPUT);
    digitalWrite(SSPin, HIGH);

    //Serial.begin(9600);
    SPI.begin();
    SPI.setDataMode(SPI_MODE0);
    SPI.setBitOrder(MSBFIRST);
    SPI.setClockDivider(SPI_CLOCK_DIV32);
 
 // The CR95HF requires a wakeup pulse on its IRQ_IN pin
 // before it will select UART or SPI mode.  The IRQ_IN pin
 // is also the UART RX pin for DIN on the BM019 board.
 
    delay(10);                      // send a wake up
    digitalWrite(IRQPin, LOW);      // pulse to put the 
    delayMicroseconds(100);         // BM019 into SPI
    digitalWrite(IRQPin, HIGH);     // mode 
    delay(10);

}


/* IDN_Command identifies the CR95HF connected to the Arduino.
This requires three steps.
1. send command
2. poll to see if CR95HF has data
3. read the response

If the correct response is received the serial monitor is used
to display the CR95HF ID number and CRC code.  This rountine is 
not that useful in using the NFC functions, but is a good way to 
verify connections to the CR95HF. 
*/
void NFCReader::IDN_Command()
 {
 byte i = 0;

// step 1 send the command
  digitalWrite(SSPin, LOW);
  SPI.transfer(0);  // SPI control byte to send command to CR95HF
  SPI.transfer(1);  // IDN command
  SPI.transfer(0);  // length of data that follows is 0
  digitalWrite(SSPin, HIGH);
  delay(1);
 
// step 2, poll for data ready
// data is ready when a read byte
// has bit 3 set (ex:  B'0000 1000')

  digitalWrite(SSPin, LOW);
  while(RXBuffer[0] != 8)
    {
    RXBuffer[0] = SPI.transfer(0x03);  // Write 3 until
    RXBuffer[0] = RXBuffer[0] & 0x08;  // bit 3 is set
    }
  digitalWrite(SSPin, HIGH);
  delay(1);


// step 3, read the data
  digitalWrite(SSPin, LOW);
  SPI.transfer(0x02);   // SPI control byte for read         
  RXBuffer[0] = SPI.transfer(0);  // response code
  RXBuffer[1] = SPI.transfer(0);  // length of data
  for (i=0;i<RXBuffer[1];i++)      
      RXBuffer[i+2]=SPI.transfer(0);  // data
  digitalWrite(SSPin, HIGH);
  delay(1);

  if ((RXBuffer[0] == 0) & (RXBuffer[1] == 15))
  {  
    Serial.println("IDN COMMAND-");  //
    Serial.print("RESPONSE CODE: ");
    Serial.print(RXBuffer[0]);
    Serial.print(" LENGTH: ");
    Serial.println(RXBuffer[1]);
    Serial.print("DEVICE ID: ");
    for(i=2;i<(RXBuffer[1]);i++)
    {
      Serial.print(RXBuffer[i],HEX);
      Serial.print(" ");
    }
    Serial.println(" ");
    Serial.print("ROM CRC: ");
    Serial.print(RXBuffer[RXBuffer[1]],HEX);
    Serial.print(RXBuffer[RXBuffer[1]+1],HEX);
    Serial.println(" ");
  }
  else
    Serial.println("BAD RESPONSE TO IDN COMMAND!");

  Serial.println(" ");
}

/* SetProtocol_Command programs the CR95HF for
ISO/IEC 15693 operation.

This requires three steps.
1. send command
2. poll to see if CR95HF has data
3. read the response

If the correct response is received the serial monitor is used
to display successful programming. 
*/
byte NFCReader::SetProtocol_Command()
 {
 byte i = 0;
 
 
// step 1 send the command
  digitalWrite(SSPin, LOW);
  SPI.transfer(0x00);  // SPI control byte to send command to CR95HF
  SPI.transfer(0x02);  // Set protocol command
  SPI.transfer(0x02);  // length of data to follow
  SPI.transfer(0x01);  // code for ISO/IEC 15693
  SPI.transfer(0x0D);  // Wait for SOF, 10% modulation, append CRC
  digitalWrite(SSPin, HIGH);
  delay(1);
 
// step 2, poll for data ready

  digitalWrite(SSPin, LOW);
  while(RXBuffer[0] != 8)
    {
    RXBuffer[0] = SPI.transfer(0x03);  // Write 3 until
    RXBuffer[0] = RXBuffer[0] & 0x08;  // bit 3 is set
    }
  digitalWrite(SSPin, HIGH);
  delay(1);

// step 3, read the data
  digitalWrite(SSPin, LOW);
  SPI.transfer(0x02);   // SPI control byte for read         
  RXBuffer[0] = SPI.transfer(0);  // response code
  RXBuffer[1] = SPI.transfer(0);  // length of data
  digitalWrite(SSPin, HIGH);

  if ((RXBuffer[0] == 0) & (RXBuffer[1] == 0))
  {
     Serial.println("PROTOCOL SET-");  //
     _NFCReady = 1; // NFC is ready
	 return _NFCReady;
  }
  else
  {
     Serial.println("BAD RESPONSE TO SET PROTOCOL");
     _NFCReady = 0; // NFC not ready
	 return _NFCReady;
  }
  //Serial.println(" ");
}

/* Inventory_Command chekcs to see if an RF
tag is in range of the BM019.

This requires three steps.
1. send command
2. poll to see if CR95HF has data
3. read the response

If the correct response is received the serial monitor is used
to display the the RF tag's universal ID.  
*/
byte NFCReader::Inventory_Command()
 {
 byte i = 0;

// step 1 send the command
  digitalWrite(SSPin, LOW);
  SPI.transfer(0x00);  // SPI control byte to send command to CR95HF
  SPI.transfer(0x04);  // Send Receive CR95HF command
  SPI.transfer(0x03);  // length of data that follows is 0
  SPI.transfer(0x26);  // request Flags byte
  SPI.transfer(0x01);  // Inventory Command for ISO/IEC 15693
  SPI.transfer(0x00);  // mask length for inventory command
  digitalWrite(SSPin, HIGH);
  delay(1);
 
// step 2, poll for data ready
// data is ready when a read byte
// has bit 3 set (ex:  B'0000 1000')

  digitalWrite(SSPin, LOW);
  while(RXBuffer[0] != 8)
    {
    RXBuffer[0] = SPI.transfer(0x03);  // Write 3 until
    RXBuffer[0] = RXBuffer[0] & 0x08;  // bit 3 is set
    }
  digitalWrite(SSPin, HIGH);
  delay(1);


// step 3, read the data
  digitalWrite(SSPin, LOW);
  SPI.transfer(0x02);   // SPI control byte for read         
  RXBuffer[0] = SPI.transfer(0);  // response code
  RXBuffer[1] = SPI.transfer(0);  // length of data
  for (i=0;i<RXBuffer[1];i++)      
      RXBuffer[i+2]=SPI.transfer(0);  // data
  digitalWrite(SSPin, HIGH);
  delay(1);

  if (RXBuffer[0] == 128) // is response code good?
  {  
    Serial.println("TAG DETECTED");
    Serial.print("UID: ");
    for(i=11;i>=4;i--)
    {
      Serial.print(RXBuffer[i],HEX);
      Serial.print(" ");
    }
    Serial.println(" ");
    _NFCReady = 2;
	return _NFCReady;
  }
  else
    {
    Serial.print("NO TAG IN RANGE - ");
    Serial.print("RESPONSE CODE: ");
    Serial.println(RXBuffer[0],HEX);
     _NFCReady = 1; //lo vuelvo a poner en espera de un TAG
	 return _NFCReady;
    }
  //Serial.println(" ");
 
}

/* Read Memory reads data from a block of memory.  
This code assumes the RF tag has less than 256 blocks
of memory and that the block size is 4 bytes.  Data
read from is displayed via the serial monitor.

This requires three steps.
1. send command
2. poll to see if CR95HF has data
3. read the response
*/
String NFCReader::Read_Memory(byte Memory_Block)
 {
 byte i = 0;
   
// step 1 send the command
  digitalWrite(SSPin, LOW);
  SPI.transfer(0x00);  // SPI control byte to send command to CR95HF
  SPI.transfer(0x04);  // Send Receive CR95HF command
  SPI.transfer(0x03);  // length of data that follows
 // SPI.transfer(0x04);  // length of data if mem > 256
  SPI.transfer(0x02);  // request Flags byte
 // SPI.transfer(0x0A);  // request Flags byte if mem > 256
  SPI.transfer(0x20);  // Read Single Block command for ISO/IEC 15693
  SPI.transfer(Memory_Block);  // memory block address
//  SPI.transfer(0x00);  // MSB of memory block address if mem > 256
  digitalWrite(SSPin, HIGH);
  delay(1);
 
// step 2, poll for data ready
// data is ready when a read byte
// has bit 3 set (ex:  B'0000 1000')

  digitalWrite(SSPin, LOW);
  while(RXBuffer[0] != 8)
    {
    RXBuffer[0] = SPI.transfer(0x03);  // Write 3 until
    RXBuffer[0] = RXBuffer[0] & 0x08;  // bit 3 is set
    }
  digitalWrite(SSPin, HIGH);
  delay(1);


// step 3, read the data
// leemos el bloque Memory_Block de tamanyo 16 bytes
  digitalWrite(SSPin, LOW);
  SPI.transfer(0x02);   // SPI control byte for read         
  RXBuffer[0] = SPI.transfer(0);  // response code
  RXBuffer[1] = SPI.transfer(0);  // length of data
  /*Serial.print("Response code and lenght: ");
  Serial.print(RXBuffer[0]); Serial.print(" ");
  Serial.print(RXBuffer[1]); Serial.print(" ");*/
  for (i=0;i<RXBuffer[1];i++)      
      RXBuffer[i+2]=SPI.transfer(0);  // data
  digitalWrite(SSPin, HIGH);
  delay(1);

  if (RXBuffer[0] == 128) // is response code good?
    {
    
    String lectura ="";
    Serial.print("Block ");
    Serial.print(Memory_Block);
    /*Serial.print(":  INT: ");
    Serial.print(RXBuffer[3]);Serial.print(" ");
    Serial.print(RXBuffer[4]);Serial.print(" ");
    Serial.print(RXBuffer[5]);Serial.print(" ");
    Serial.print(RXBuffer[6]);Serial.print(" ");
    Serial.print(RXBuffer[7]);Serial.print(" ");
    Serial.print(RXBuffer[8]);Serial.print(" ");
    Serial.print(RXBuffer[9]);Serial.print(" ");
    Serial.print(RXBuffer[10]);Serial.print(" ");*/
    Serial.print(" -  HEX:  ");
      lectura += printHex(RXBuffer[3],2);
      lectura += printHex(RXBuffer[4],2);
      lectura += printHex(RXBuffer[5],2);
      lectura += printHex(RXBuffer[6],2);
      lectura += printHex(RXBuffer[7],2);
      lectura += printHex(RXBuffer[8],2);
      lectura += printHex(RXBuffer[9],2);
      lectura += printHex(RXBuffer[10],2);
      /*Serial.print(RXBuffer[3],HEX);Serial.print(" ");
      Serial.print(RXBuffer[4],HEX);Serial.print(" ");
      Serial.print(RXBuffer[5],HEX);Serial.print(" ");
      Serial.print(RXBuffer[6],HEX);Serial.print(" ");
      Serial.print(RXBuffer[7],HEX);Serial.print(" "); 
      Serial.print(RXBuffer[8],HEX);Serial.print(" ");
      Serial.print(RXBuffer[9],HEX);Serial.print(" ");
      Serial.print(RXBuffer[10],HEX);*/
    //Serial.print(" - en cadena:  ");
    /*lectura += String(RXBuffer[3],HEX);
    lectura += String(RXBuffer[4],HEX);
    lectura += String(RXBuffer[5],HEX);
    lectura += String(RXBuffer[6],HEX);
    lectura += String(RXBuffer[7],HEX);
    lectura += String(RXBuffer[8],HEX);
    lectura += String(RXBuffer[9],HEX);
    lectura += String(RXBuffer[10],HEX);*/
     
    _NFCReady = 2;
    return lectura;
    }
  else
    {
    Serial.print("Read Memory Block Command FAIL");Serial.println(" ");
    _NFCReady = 1; //lo vuelvo a poner en espera de un TAG
    }
 }
 
 // la funcion GetTime se utiliza para calcular el tiempo de uso del sensor
 void NFCReader::GetTime(int minutes)
 {
   int tdias= minutes/1440;
   int ttemp=minutes-(tdias*1440);
   int thoras=(ttemp/60);
   int tmin=ttemp-(thoras*60);
   Serial.print("Este sensor esta activo durante ");Serial.print(tdias,DEC); 
   Serial.print(" dias, ");Serial.print(thoras,DEC);
   Serial.print(" horas, ");Serial.print(tmin,DEC);
   Serial.println(" minutos."); 
  
 }
 
 //la funcion GetGlucose tiene la regresion para el calculo de la glicemia
 int NFCReader::GetGlucose(int readGlucose)
 {
    int mascara=0x0FFF;
    //byte temp=readGlucose;
   int glucose = ((readGlucose & mascara)/6)-37;
   return glucose;
 }
 
 
 //la funcion printHex es necesaria para evitar que arduino elimine el primer cero
 // en los HEX, ya que por defecto lo quita si se usa Serial.print(var, HEX)
 String printHex(int num, int precision) {
      char tmp[16];
      char format[128];
       //sprintf(format, "0x%%.%dX", precision);
      sprintf(format, "%%.%dX", precision);

      sprintf(tmp, format, num);
      //Serial.print(tmp); Serial.print(" ");
      return tmp;
 }
